#!/usr/bin/perl

# Written by Jason Rennie (jr6b@andrew.cmu.edu)

# This script will search through all your current EXMH e-mail messages and
#   create an .idata accordingly.  This way, instead of needing to refile a
#   number of messages before ifile knows your preferences, ifile will have
#   an instant knowledge base of your interests

# NOTE: This script will destroy your current .idata file (if one exists).
#       A backup will be created as ".idata~".
#       If you do not keep extensive amounts of old mail, this will create a
#       more current version of your preferences.  If you keep old mail, the
#       new .idata file will include all the information in the old .idata
#       file (and then some).

# Any directory which contains a .skip_me file will be excluded from the
#   knowledge base.  Your drafts folder (as defined in your .mh_profile)
#   will be automatically skipped.

# This can be a very cpu/memory intensive script, especially if you have more
#   than just a few hundred messages.  My 1800 messages took 6 minutes on
#   my P100 (48 megs ram).

# Running of this script will likely result in a larger ~/.idata file.
#   However, the size should remain fairly stable after this script is run.

open(TMP, ">> /tmp/knowledge_base.info")
    || warn "Could not open /tmp/knowledge_base.info: $!\n";

print TMP "knowledge_base.perl called\n";

$home_dir = $ENV{"HOME"}."/";

print TMP "Home dir = $home_dir\n";

open(MH, "< $home_dir.mh_profile") 
    || die "No .mh_profile found: $!\n";

while (<MH>) {
    if (m/Path:\s*(\w+)/) {
	$mail_path = $1;
	$mail_path .= "/";
    }
    if (m/draft-folder:\s*(\w+)/) {
	$draft_folder = $1;
    }
}

print TMP "Mail Path = $mail_path\n";

print TMP "cp $home_dir.idata $home_dir.idata~\n";
system "cp $home_dir.idata $home_dir.idata~";

$tv[0] = 0;
$word[0] = "";
$tot_words = 0;

# Gets a listing of all the folders which contain messages

open(FOLDERS, "< $home_dir${mail_path}.folders")
    || &ouch("Couldn't find .folders\n");

while ($folder = <FOLDERS>)
{
    chop $folder;
    $tv[@tv] = $folder;
    $dest_folder = $folder;

    print TMP "Reading $home_dir$mail_path$folder/\n";
    print "Reading $home_dir$mail_path$folder/\n";

    if (opendir(BOX, "$home_dir$mail_path$folder/")
	&& ! -f "$home_dir$mail_path$folder/.skip_me"
	&& $folder ne $draft_folder)
    {
	while ($message = readdir(BOX))
	{
	    # checks to make sure it isn't dot-file or deleted file
	    # also doesn't consider files > 30,000 bytes (likely to contain
	    #   encoded attachements)
	    if ($message =~ m/^[^,\.]/ && $message =~ m/[^~]$/ 
		&& -f "$home_dir$mail_path$folder/$message"
		&& -s "$home_dir$mail_path$folder/$message" < 30000)
	    {
		open(MSG, "$home_dir$mail_path$folder/$message")
		    || print TMP "Unable to open $home_dir$mail_path$folder/$message: $!\n";

		print TMP "$message ";

		local($line);

		while ($line = <MSG>)
		{
		    $weight = 1;
		    if ($line =~ s/subject://i) { $weight = 3; }
		    if ($line =~ s/from://i) { $weight = 3; }
		    
		    while ($line =~ s/([A-Za-z]+)[^A-Za-z]+//) 
		    {
			local($wrd);
			$wrd = $1;
			# get rid of tiny words
			if (length($wrd) > 2) 
			{
			    $wrd =~ tr/A-Z/a-z/;
			    if ($index{$wrd} == 0) 
			    {
				$word[@word] = $wrd;
				$index{$wrd} = $#word;
				$nk{$wrd.",".$dest_folder} += $weight;
				$sum_nk{$wrd}++;
			    } else 
			    {
				$nk{$wrd.",".$dest_folder} += $weight;
				$sum_nk{$wrd}++;
			    }
			    $n{$dest_folder} += $weight;
			}
		    }
		}
		close(MSG);
		$docs{$dest_folder}++;

		for ($i=1; $i <= $#word; $i++) 
		{
		    $age{$word[$i]}++;

		    # if the word is old and infrequent, remove it
		    if (($age{$word[$i]} >= 16) && ($sum_nk{$word[$i]} <= 3))
		    {
			foreach $target (@tv) {
			    delete $nk{$word[$i].",".$target};
			}
			delete $age{$word[$i]};
			delete $index{$word[$i]};
			delete $sum_nk{$word[$i]};

			$word[$i] = $word[$#word];
			$index{$word[$i]} = $i;
			$#word--;
			$i--;
		    }
		}
	    }
	}
	print TMP "\n";
	closedir(BOX);
    } else {
	if (-f "$home_dir$mail_path$folder/.skip_me" ||
	    $draft_folder eq $folder) {
	    print TMP "Skipping $home_dir$mail_path$folder/\n";
	    print "Skipping $home_dir$mail_path$folder/\n";
	} else {
	    print TMP "Did not open $home_dir${mail_path}$folder/: $!\n";
	    print "Did not open $home_dir${mail_path}$folder/: $!\n";
	}
    }
}

open(DATA, "> $home_dir.idata")
    || &ouch("Couldn't open data file\n");

# prints folder names
for ($i=1; $i <= $#tv; $i++) 
{
    print DATA $tv[$i]." ";
}
print DATA "\n";

# prints total word frequency values
for ($i=1; $i <= $#tv; $i++) 
{
    if ($n{$tv[$i]} > 0) {
	print DATA $n{$tv[$i]}." ";
    } else {
	print DATA "0 ";
    }
}
print DATA "\n";

# prints number of documents for each category
for ($i=1; $i <= $#tv; $i++) 
{
    if ($n{$tv[$i]} > 0) {
	print DATA $docs{$tv[$i]}." ";
    } else {
	print DATA "0 ";
    }
}
print DATA "\n";

for ($i=1; $i <= $#word; $i++) 
{
    print DATA $word[$i]." ".$age{$word[$i]}." ";
    
    for ($j=1; $j <= $#tv; $j++)
    {
	if ($nk{$word[$i].",".$tv[$j]} > 0)
	{
	    print DATA $j.":".$nk{$word[$i].",".$tv[$j]}." ";
	}
    }
    print DATA "\n";
}

close(DATA);

################

sub ouch {
    local($string) = @_;
    print TMP $string."\n";
    die "$string.: $!\n";
}
